package cn.com.qmth.examcloud.core.reports.api.controller;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamCourseMainTopTen;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamCourseReportMainBean;
import cn.com.qmth.examcloud.core.reports.base.enums.ReportStatus;
import cn.com.qmth.examcloud.core.reports.dao.entity.ProjectEntity;
import cn.com.qmth.examcloud.core.reports.service.ExamCourseDataReportService;
import cn.com.qmth.examcloud.core.reports.service.ProjectService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "考试课程报表")
@RequestMapping("${$rmp.ctr.reports}/" + "examCourseDataReport")
public class ExamCourseDataReportController extends ControllerSupport {
	@Autowired
	private ExamCourseDataReportService examCourseDataReportService;
	@Autowired
	private ProjectService projectService;

	@ApiOperation(value = "查询考试课程数据")
	@GetMapping(value = "/getExamCourseMainList")
	public List<ExamCourseReportMainBean> getExamCourseMainList(@RequestParam Long projectId,
			@RequestParam(required = false) Long examId) {
		if (projectId == null) {
			throw new StatusException("100011", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100012", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100013", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100014", "只能查看报表计算成功的项目");
		}
		return examCourseDataReportService.getExamCourseMainList(projectId, examId);
	}

	@ApiOperation(value = "查询考试课程数据Top10")
	@GetMapping(value = "/getExamCourseMainTop10")
	public ExamCourseMainTopTen getExamCourseMainTop10(@RequestParam Long projectId,
			@RequestParam(required = false) Long examId) {
		if (projectId == null) {
			throw new StatusException("100021", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100022", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100023", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100024", "只能查看报表计算成功的项目");
		}
		return examCourseDataReportService.getExamCourseMainTop10(projectId, examId);
	}

	@ApiOperation(value = "导出数据")
	@GetMapping(value = "/export")
	public void exportAll(@RequestParam Long projectId, @RequestParam(required = false) Long examId,
			HttpServletResponse response) {
		if (projectId == null) {
			throw new StatusException("100031", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100032", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100033", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100034", "只能导出报表计算成功的项目");
		}
		examCourseDataReportService.exportAll(pe, examId, response);
	}
}