package cn.com.qmth.examcloud.core.reports.api.controller;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.reports.base.bean.AddProjectReq;
import cn.com.qmth.examcloud.core.reports.base.bean.ProjectBean;
import cn.com.qmth.examcloud.core.reports.base.bean.QueryProjectPageReq;
import cn.com.qmth.examcloud.core.reports.base.bean.UpdateProjectReq;
import cn.com.qmth.examcloud.core.reports.base.bean.UpdateProjectScoreReq;
import cn.com.qmth.examcloud.core.reports.base.enums.DataOrigin;
import cn.com.qmth.examcloud.core.reports.service.ProjectService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@Api(tags = "项目列表")
@RequestMapping("${$rmp.ctr.reports}/" + "project")
public class ProjectController extends ControllerSupport {
	@Autowired
	private ProjectService projectService;

	@ApiOperation(value = "查询已结束的考试")
	@GetMapping(value = "/allExam")
	public List<ExamBean> queryAllExam() {
		User user = getAccessUser();
		List<ExamBean> list = projectService.queryAllExam(user.getRootOrgId());
		return list;
	}

	@DeleteMapping("/{id}")
	@ApiOperation(value = "删除项目")
	public void delProject(@PathVariable Long id) {
		User user = getAccessUser();
		projectService.delProject(id, user.getRootOrgId());
	}

	@PostMapping("/add")
	@ApiOperation(value = "新增项目")
	public Long addProject(@RequestBody @Valid AddProjectReq req) {
		if (req.getDataOrigin().equals(DataOrigin.SYNC) && (req.getExamIds() == null || req.getExamIds().size() == 0)) {
			throw new StatusException("1000001", "请选择考试");
		}
		if (req.getDataOrigin().equals(DataOrigin.IMPORT)) {
			throw new StatusException("1000002", "暂不支持导入");
		}
		User user = getAccessUser();
		return projectService.addProject(req, user.getRootOrgId());
	}

	@PostMapping("/update")
	@ApiOperation(value = "修改项目")
	public void updateProject(@RequestBody @Valid UpdateProjectReq req) {
		User user = getAccessUser();
		projectService.updateProject(req, user.getRootOrgId());
	}

	@PutMapping("/enable")
	@ApiOperation(value = "启用")
	public void enable(
			@RequestParam(required = true) @NotBlank(message = "请选择项目") @ApiParam(value = "项目Id集合") String ids) {
		User user = getAccessUser();
		projectService.updateEnable(ids, true, user.getRootOrgId());
	}

	@PutMapping("/disable")
	@ApiOperation(value = "禁用")
	public void disable(
			@RequestParam(required = true) @NotBlank(message = "请选择项目") @ApiParam(value = "项目Id集合") String ids) {
		User user = getAccessUser();
		projectService.updateEnable(ids, false, user.getRootOrgId());
	}

	@GetMapping("page/{pageNo}/{pageSize}")
	@ApiOperation(value = "项目列表分页查询")
	public PageInfo<ProjectBean> queryPage(@ModelAttribute QueryProjectPageReq req,
			@PathVariable @ApiParam(value = "pageNo = 1,2,3...") Integer pageNo, @PathVariable Integer pageSize) {
		User user = getAccessUser();
		return projectService.queryPage(req, pageNo, pageSize, user.getRootOrgId());
	}

	@PostMapping("/compute/{id}")
	@ApiOperation(value = "开始计算")
	public void addReportsCompute(@PathVariable Long id) {
		User user = getAccessUser();
		projectService.addReportsCompute(id, user.getRootOrgId());
	}

	@PostMapping("/updateScore")
	@ApiOperation(value = "修改项目分数")
	public void updateScore(@RequestBody @Valid UpdateProjectScoreReq req) {
		User user = getAccessUser();
		projectService.updateScore(req, user.getRootOrgId());
	}

	@GetMapping("{id}")
	@ApiOperation(value = "查询项目")
	public ProjectBean getProject(@PathVariable @ApiParam(value = "id") Long id) {
		User user = getAccessUser();
		return projectService.getProject(id, user.getRootOrgId());
	}
    @ApiOperation(value="批量获取评项目")
	@GetMapping("/getProjectList")
	public List<ProjectBean> getProjectList(@RequestParam(required = true) String ids) {
    	List<Long> pids = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
    	User user = getAccessUser();
		return projectService.getProjectList(pids, user.getRootOrgId());
	}
}