package cn.com.qmth.examcloud.core.reports.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.reports.base.bean.StudentCumulativeCountBean;
import cn.com.qmth.examcloud.core.reports.base.util.online.ActiveDataUtil;
import cn.com.qmth.examcloud.core.reports.base.util.online.OnlineCount;
import cn.com.qmth.examcloud.core.reports.dao.StudentCumulativeCountRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.StudentCumulativeCountEntity;
import cn.com.qmth.examcloud.core.reports.service.StudentCumulativeCountService;

@Service
public class StudentCumulativeCountServiceImpl implements StudentCumulativeCountService {
	@Autowired
	private StudentCumulativeCountRepo studentCumulativeCountRepo;
	@Autowired
	private JdbcTemplate jdbcTemplate;

	@Transactional
	@Override
	public synchronized void saveCount() {
		// 取当前人数累计数据
		List<OnlineCount> ret = ActiveDataUtil.getStudentLoginCount();
		// 判断StudentLoginDataReportDay跟当前系统日期是否一致，不一致说明到了第二天，累计数据需要清空
		Date now = new Date();
		Date last = DateUtils.addDays(now, -1);
		String lastday = DateUtil.format(last, DateUtil.DatePatterns.YYYY_MM_DD);
		String today = DateUtil.format(now, DateUtil.DatePatterns.YYYY_MM_DD);
		boolean isTomorrow = !today.equals(ActiveDataUtil.getStudentLoginDataReportDay());
		if (isTomorrow) {
			ActiveDataUtil.clearStudentLoginCount();
		}

		if (ret != null) {
			for (OnlineCount oc : ret) {
				addOrUpdateCount(oc, lastday, today, isTomorrow);
			}
		}
	}

	private void addOrUpdateCount(OnlineCount oc, String lastday, String today, boolean isTomorrow) {
		if (isTomorrow) {
			// 第二天
			StudentCumulativeCountEntity oe = studentCumulativeCountRepo.findByRootOrgIdAndReportDay(oc.getRootOrgId(),
					lastday);
			if (oe != null) {
				// 更新昨天的数据
				oe.setTotalCount(oc.getOnlineCount());
				studentCumulativeCountRepo.save(oe);
			}
			// 插入一条人数为零的今天数据
			StudentCumulativeCountEntity e = new StudentCumulativeCountEntity();
			e.setRootOrgId(oc.getRootOrgId());
			e.setTotalCount(0);
			e.setReportDay(today);
			Date now = new Date();
			e.setCreationTime(now);
			e.setUpdateTime(now);
			studentCumulativeCountRepo.save(e);
		} else {
			// 今天 插入或更新一条当前人数的今天数据
			StudentCumulativeCountEntity e = studentCumulativeCountRepo.findByRootOrgIdAndReportDay(oc.getRootOrgId(),
					today);
			if (e == null) {
				e = new StudentCumulativeCountEntity();
				e.setRootOrgId(oc.getRootOrgId());
				e.setReportDay(today);
				Date now = new Date();
				e.setCreationTime(now);
				e.setUpdateTime(now);
			}
			e.setTotalCount(oc.getOnlineCount());
			studentCumulativeCountRepo.save(e);
		}
	}

	@Override
	public List<StudentCumulativeCountBean> getLastNdayOnlineCount(Integer nday) {
		if (nday < 1 || nday > 100) {
			throw new StatusException("100001", "查询天数1到100之间");
		}
		List<String> ds=getDays(nday);
		StringBuffer sb = new StringBuffer();
		sb.append(" select sum(t.total_count) totalCount,t.report_day reportDay from ec_r_student_cumulative_count t where t.report_day in ('-1'");
		for (String d:ds) {
			sb.append(",'").append(d).append("'");
		}
		sb.append(") group by t.report_day");
		RowMapper<StudentCumulativeCountBean> rowMapper=new BeanPropertyRowMapper<StudentCumulativeCountBean>(StudentCumulativeCountBean.class);
		List<StudentCumulativeCountBean> ret = jdbcTemplate.query(sb.toString(), rowMapper);
		
		List<StudentCumulativeCountBean> initList=getInitList(ds);
		if (CollectionUtils.isNotEmpty(ret)) {
			Map<String, Integer> map = ret.stream().collect(Collectors.toMap(StudentCumulativeCountBean::getReportDay,
					account -> account.getTotalCount(), (key1, key2) -> key2));
			for(StudentCumulativeCountBean b:initList) {
				Integer c=map.get(b.getReportDay());
				b.setTotalCount(c==null?0:c);
			}
		}
		return initList;
	}
	private List<StudentCumulativeCountBean> getInitList(List<String> ds) {
		List<StudentCumulativeCountBean> ret = Lists.newArrayList();
		for(String d:ds) {
			StudentCumulativeCountBean sb=new StudentCumulativeCountBean();
			sb.setReportDay(d);
			sb.setTotalCount(0);
			ret.add(sb);
		}
		return ret;
	}
	private List<String> getDays(Integer nday) {
		List<String> ds = Lists.newArrayList();
		Date now = new Date();
		for (int i = nday; i > 0; i--) {
			Date day = DateUtils.addDays(now, 1-i);
			String daystr = DateUtil.format(day, DateUtil.DatePatterns.YYYY_MM_DD);
			ds.add(daystr);
		}
		return ds;
	}
}
